// ============================================================================
//
// Copyright (c) 2010, heig-vd.ch, niceideas.ch - Jerome Kehrli
//
// You may distribute this code under the terms of the GNU LGPL license
// (http://www.gnu.org/licenses/lgpl.html). [^]
//
// ============================================================================


package ch.niceideas.communityBoard.data;

import java.io.Serializable;
import javax.persistence.*;

import java.sql.Timestamp;
import java.util.List;


/**
 * The persistent class for the CB_NOTE database table.
 * <p />
 * 
 * The Note entity represents a note as it is stored in the database. A note 
 * is posted by a specific user on a specific forum. The user should provide a 
 * title and the content of the note. The note post time is recorded 
 * automatically by the database.
 */
@Entity
@Table(name="APP.CB_NOTE")
public class Note implements Serializable {
    
	private static final long serialVersionUID = 1L;
	private int id;
	private String content;
	private Timestamp posted = new Timestamp(System.currentTimeMillis());
	private String title;
	private Forum forum;
	private User user;
	private List<Rating> ratings;

    public Note() {
    }

    /**
     * We use the preRemove hook to unregister the note from the list of notes
     * belonging to the user and the list of notes from the forum.  
     */
    @PreRemove
    public void preRemove() {
        getForum().getNotes().remove(this);
        getUser().getNotes().remove(this);
        //setForum(null);
    }

	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	@Column(unique=true, nullable=false)
	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}


	@Column(nullable=false, length=1)
	public String getContent() {
		return this.content;
	}

	public void setContent(String content) {
		this.content = content;
	}

	@Column(nullable=false)
	public Timestamp getPosted() {
		return this.posted;
	}

	public void setPosted(Timestamp posted) {
		this.posted = posted;
	}

	@Column(length=256)
	public String getTitle() {
		return this.title;
	}

	public void setTitle(String title) {
		this.title = title;
	}


	//bi-directional many-to-one association to Forum
	@ManyToOne(fetch=FetchType.LAZY)
	@JoinColumn(name="FORUM_ID", nullable=false)
	public Forum getForum() {
		return this.forum;
	}

	public void setForum(Forum forum) {
		this.forum = forum;
		if (forum != null) {
		    forum.refreshNote (this);
		}
	}
	

	//bi-directional many-to-one association to User
	@ManyToOne(fetch=FetchType.LAZY)
	@JoinColumn(name="USER_ID", nullable=false)
	public User getUser() {
		return this.user;
	}

	public void setUser(User user) {
		this.user = user;
		if (user != null) {
            user.refreshNote (this);
        }
	}
	

	//bi-directional many-to-one association to Opinion
	@OneToMany(mappedBy="note", cascade={CascadeType.MERGE,CascadeType.PERSIST,CascadeType.REFRESH,CascadeType.REMOVE})
	public List<Rating> getRatings() {
		return this.ratings;
	}

	public void setRatings(List<Rating> ratings) {
		this.ratings = ratings;
	}
	
	/**
	 * 
	 * @param rating
	 */
	public void addRating(Rating rating) {
	    getRatings().add(rating);
	}
	
	public boolean isWithRatings() {
		return this.getRatings().size() > 0;
	}
	
	/**
	 * TODO Document me 
	 * 
	 * @return
	 */
	public int getGlobalRating() {
		int globalRating = 0;
		int ratingCount = 0;
		for (Rating rating : this.getRatings()) {
			ratingCount++;
			globalRating += rating.getRating();
		}
		return ratingCount > 0 ? globalRating / ratingCount : 0;
	}

    public void refreshRating(Rating rating) {
        for (int i = 0; i < getRatings().size(); i++) {
            Rating cur = getRatings().get(i);
            if (cur.getId() == rating.getId()) {
                getRatings().remove(i);
                break;
            }
        }
        getRatings().add(rating);
    }
	
}